#!/usr/bin/csh -f

if ("clean" == $1) then
    echo 'removing generated files'
    rm -f raw.csv aug.csv data.csv R.aug outl0.csv outl.csv R.outl \
          params raw.eps models.gp models.eps outl.eps omodels.gp \
          omodels.eps mean_median.eps run.out resid.aug.LSQ \
          resid.aug.LAR resid.outl.LSQ resid.outl.LAR resid.outl.eps \
          resid_outl_cdf.eps resid_outl_neg_cdf.eps
    exit
endif

echo ''
echo 'generate mean/median plot'
rm -f mean_median.eps
gnuplot mean_median.gnu

echo ''
echo 'generate synthetic data set'
rm -f raw.csv
./gen.awk > raw.csv

echo ''
echo 'transform raw data to prepare for R by augmenting with sine(time) field'
rm -f aug.csv
./xfrm.awk < raw.csv > aug.csv

echo ''
echo 'run R to fit least-squares and LAR models'
rm -f data.csv R.aug
ln -s aug.csv data.csv
R -s --no-save < cmds.R >& R.aug
rm -f data.csv

echo ''
echo 'extract residuals in plot-friendly form'
rm -f resid.aug.LSQ resid.aug.LAR
gawk '/LSQ_residuals_begin/,/LSQ_residuals_end/' R.aug \
    | grep -v LSQ | gawk '0==FNR%2' \
    | gawk '{for(i=1;i<=NF;i++)print n++, $i}' > resid.aug.LSQ
gawk '/LAR_residuals_begin/,/LAR_residuals_end/' R.aug \
    | grep -v LAR | gawk '{for(i=2;i<=NF;i++)print n++, $i}' > resid.aug.LAR

echo ''
echo 'extract model parameters'
echo 'raw LSQ' `grep -A2 LSQ_coeff R.aug | tail -n 1` >  params
echo 'raw LAR' `grep -A2 LAR_coeff R.aug | tail -n 1` >> params

echo ''
echo 'add outliers to raw data and augment with sine(time)'
rm -f outl0.csv outl.csv
./outl.awk < raw.csv > outl0.csv
./xfrm.awk < outl0.csv > outl.csv

echo ''
echo 'run R to fit least-squares and LAR models to data with outliers'
rm -f data.csv R.outl
ln -s ./outl.csv ./data.csv
R -s --no-save < cmds.R >& R.outl
rm -f data.csv

echo ''
echo 'extract residuals in plot-friendly form'
rm -f resid.outl.LSQ resid.outl.LAR
gawk '/LSQ_residuals_begin/,/LSQ_residuals_end/' R.outl \
    | grep -v LSQ | gawk '0==FNR%2' \
    | gawk '{for(i=1;i<=NF;i++)print n++, $i}' > resid.outl.LSQ
gawk '/LAR_residuals_begin/,/LAR_residuals_end/' R.outl \
    | grep -v LAR | gawk '{for(i=2;i<=NF;i++)print n++, $i}' > resid.outl.LAR

echo ''
echo 'plot residuals for data with outliers'
rm -f resid.outl.eps
gnuplot resid.gnu
rm -f   resid_outl_cdf.eps
gnuplot resid_outl_cdf.gnu
rm -f   resid_outl_neg_cdf.eps
gnuplot resid_outl_neg_cdf.gnu

echo ''
echo 'extract model parameters'
echo 'out LSQ' `grep -A2 LSQ_coeff R.outl | tail -n 1` >> params
echo 'out LAR' `grep -A2 LAR_coeff R.outl | tail -n 1` >> params

echo ''
echo 'model parameters (intercept, slope, magnitude of sinewave):'
cat params

echo ''
echo 'generate plots of raw data and models'

rm -f raw.eps
cat template.gnuplot \
| sed 's/PLOT_LABEL1//g' \
| sed 's/PLOT_LABEL2//g' \
| sed 's/PLOT_LABEL3//g' \
| sed 's/PLOT_LABEL4//g' \
| sed 's/PLOT_EPS/raw.eps/g' \
| sed 's/PLOT_DATA/raw.csv/g' | gnuplot

set rlsqi = `gawk '/raw LSQ/{print $3}' params`
set rlsqs = `gawk '/raw LSQ/{print $4}' params`
set rlsqa = `gawk '/raw LSQ/{print $5}' params`

set rlari = `gawk '/raw LAR/{print $3}' params`
set rlars = `gawk '/raw LAR/{print $4}' params`
set rlara = `gawk '/raw LAR/{print $5}' params`

rm -f models.gp models.eps
cat template.gnuplot \
| sed 's/PLOT_LABEL1//g' \
| sed 's/PLOT_LABEL2//g' \
| sed 's/PLOT_LABEL3//g' \
| sed 's/PLOT_LABEL4//g' \
| sed 's/PLOT_EPS/models.eps/g' \
| sed 's/PLOT_DATA/raw.csv/g' > models.gp
echo ' t "data"' ", [0:] $rlsqi + $rlsqs * x + $rlsqa * sin((x/24)*2*pi) w lines \" >> models.gp
echo 'lt 1 lc rgb "red" t "LSQ"' >> models.gp
# no need for LAR fit; it lies atop LSQ
# echo "[0:] $rlari + $rlars * x + $rlara * sin((x/24)*2*pi) w lines \" >> models.gp
# echo 'lt 1 lc rgb "blue" t "LAR" \' >> models.gp

gnuplot models.gp

echo ''
echo 'generate plots of data and models with outliers'

rm -f outl.eps
cat template.gnuplot \
| sed 's/PLOT_LABEL1//g' \
| sed 's/PLOT_LABEL2//g' \
| sed 's/PLOT_LABEL3//g' \
| sed 's/PLOT_LABEL4//g' \
| sed 's/PLOT_EPS/outl.eps/g' \
| sed 's/PLOT_DATA/outl0.csv/g' | gnuplot

set olsqi = `gawk '/out LSQ/{print $3}' params`
set olsqs = `gawk '/out LSQ/{print $4}' params`
set olsqa = `gawk '/out LSQ/{print $5}' params`

set olari = `gawk '/out LAR/{print $3}' params`
set olars = `gawk '/out LAR/{print $4}' params`
set olara = `gawk '/out LAR/{print $5}' params`

rm -f omodels.gp omodels.eps
cat template.gnuplot \
| sed 's/PLOT_LABEL1/set label "LAR fit" at -2,8300 textcolor rgb "blue"/g' \
| sed 's/PLOT_LABEL2/set label "Least-squares fit" at 24,16000 textcolor rgb "red"/g' \
| sed 's/PLOT_LABEL3/set label "overload" at 20,25000/g' \
| sed 's/PLOT_LABEL4/set label "outage" at 155,3000/g' \
| sed 's/PLOT_EPS/omodels.eps/g' \
| sed 's/PLOT_DATA/outl0.csv/g' > omodels.gp
echo ' t ""' ", [0:] $olsqi + $olsqs * x + $olsqa * sin((x/24)*2*pi) w lines \" >> omodels.gp
echo 'lt 1 lc rgb "red" t "Least Squares", \' >> omodels.gp
echo "[0:] $olari + $olars * x + $olara * sin((x/24)*2*pi) w lines \" >> omodels.gp
echo 'lt 1 lc rgb "blue" t "LAR" \' >> omodels.gp

gnuplot omodels.gp

echo ''

